--------------------------------------------------------------------------------
--
-- File: shuriken.vhd
-- Version: 1.1 Start date 8/9/2014 (Steven Taffs)
--
-- Description: 
--
-- Loads Shuriken OS from serial flash onto flipper exi data in line
--
-- Targeted device: CPLD  xc9572xl-10vq44
-- Author: Steven Taffs
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

----------------------------------------------------------------
-- Main entity
----------------------------------------------------------------
entity shuriken is
    Port ( 
	        --gamecube connections
	        exi_serial_in : in  STD_LOGIC;
           exi_serial_to_gc : out  STD_LOGIC;
           exi_clk : in  STD_LOGIC;
           exi_cs : in  STD_LOGIC;
			  --serial flash connections
			  serial_flash_in : out  STD_LOGIC;
			  serial_flash_out : in  STD_LOGIC;
			  serial_flash_cs : out  STD_LOGIC  
         );			  
end shuriken;

architecture arch_shuriken of shuriken is

----------------------------------------------------------------
-- signal defines
----------------------------------------------------------------

signal bit_counter: std_logic_vector(13 downto 0);    --counts number of bits sent
signal serial_payload_enable: std_logic;              --send payload to gc if this gets set
signal serial_payload_disable: std_logic := '0';      --stop payload to gc if this gets set

--M25P40 serial flash chips read op-code
signal read_cmd: std_logic_vector(6 downto 0) := "0000011";
--serial flash read address counter (high / middle / low bytes)
signal serial_addr_counter: std_logic_vector(14 downto 0) := (others => '0');
--enables serial flash chip select line
signal serial_enable: std_logic := '0';

----------------------------------------------------------------
-- asynchronous events
----------------------------------------------------------------	
begin
   --control data to gc 
	exi_serial_to_gc <= ((serial_flash_out and serial_payload_enable) or (exi_serial_in and not serial_payload_enable))
		when (exi_cs = '0') else '0';
	
	--control serial flash chip select line
	serial_flash_cs <= exi_cs when (serial_enable = '1') else 'Z';
	
----------------------------------------------------------------
-- synchronised processes 
----------------------------------------------------------------					
   process (exi_cs, exi_clk)
		begin
		if (exi_cs = '1') then
			bit_counter <= (others => '0');
			serial_payload_enable <= '0';
			serial_flash_in <= 'Z';
		else
			if (exi_clk'event and exi_clk='1' and serial_payload_disable = '0') then
			
				if(serial_addr_counter = "000000000000000") then -- first transfer start from byte 32
			
					case bit_counter is				
						when "00000011111111" =>                  --	time to select serial flash chip
							 serial_enable <= '1';		         		
							 
						when "00000100000000" =>
							 serial_flash_in <= read_cmd(6);				 
						
						when "00000100000001" =>
							 serial_flash_in <= read_cmd(5);
							 
						when "00000100000010" =>
							 serial_flash_in <= read_cmd(4);
							 
						when "00000100000011" =>
							 serial_flash_in <= read_cmd(3);
							 
						when "00000100000100" =>
							 serial_flash_in <= read_cmd(2);
							 
						when "00000100000101" =>
							 serial_flash_in <= read_cmd(1);
							 
						when "00000100000110" =>
							 serial_flash_in <= read_cmd(0);		
							 
						when "00000100001011" =>	                 -- send high / middle / low address counter value
							 serial_flash_in <= serial_addr_counter(14);
							 
						when "00000100001100" =>	                   
							 serial_flash_in <= serial_addr_counter(13);
							 
						when "00000100001101" =>	                   
							 serial_flash_in <= serial_addr_counter(12);
							 
						when "00000100001110" =>	                    
							 serial_flash_in <= serial_addr_counter(11);
							 
						when "00000100001111" =>	                    
							 serial_flash_in <= serial_addr_counter(10);
							 
						when "00000100010000" =>	                   
							 serial_flash_in <= serial_addr_counter(9);
							 
						when "00000100010001" =>	                   
							 serial_flash_in <= serial_addr_counter(8);
							 
						when "00000100010010" =>	                   
							 serial_flash_in <= serial_addr_counter(7);
							 
						when "00000100010011" =>	                   
							 serial_flash_in <= serial_addr_counter(6);
							 
						when "00000100010100" =>	                   
							 serial_flash_in <= serial_addr_counter(5);
							 
						when "00000100010101" =>	                   
							 serial_flash_in <= serial_addr_counter(4);
							 
						when "00000100010110" =>	                   
							 serial_flash_in <= serial_addr_counter(3);

                  when "00000100010111" =>	                   
							 serial_flash_in <= serial_addr_counter(2);	

						when "00000100011000" =>	                   
							 serial_flash_in <= serial_addr_counter(1);	

						when "00000100011001" =>	                   
							 serial_flash_in <= serial_addr_counter(0);										 
												
						when "00000100011111" =>            -- time to send payload (switch to serial flash output)
							 serial_payload_enable <= '1';

                  when "10000000011111" =>            -- inc address counter (1024 - 32)                        
							 serial_addr_counter <= serial_addr_counter + "11111"; 					 
							 
						when others =>
							 serial_flash_in <= '0';         -- set read address lower 5 bits to zero

					end case;
				
				else  -- all other transfers start at byte 4
				
				   case bit_counter is											 
						when "00000000000000" =>
							 serial_flash_in <= read_cmd(6);				 
						
						when "00000000000001" =>
							 serial_flash_in <= read_cmd(5);
							 
						when "00000000000010" =>
							 serial_flash_in <= read_cmd(4);
							 
						when "00000000000011" =>
							 serial_flash_in <= read_cmd(3);
							 
						when "00000000000100" =>
							 serial_flash_in <= read_cmd(2);
							 
						when "00000000000101" =>
							 serial_flash_in <= read_cmd(1);
							 
						when "00000000000110" =>
							 serial_flash_in <= read_cmd(0);
							 
						when "00000000001011" =>	                 -- send high / middle / low address counter value
							 serial_flash_in <= serial_addr_counter(14);
							 
						when "00000000001100" =>	                   
							 serial_flash_in <= serial_addr_counter(13);
							 
						when "00000000001101" =>	                   
							 serial_flash_in <= serial_addr_counter(12);
							 
						when "00000000001110" =>	                    
							 serial_flash_in <= serial_addr_counter(11);
							 
						when "00000000001111" =>	                    
							 serial_flash_in <= serial_addr_counter(10);
							 
						when "00000000010000" =>	                   
							 serial_flash_in <= serial_addr_counter(9);
							 
						when "00000000010001" =>	                   
							 serial_flash_in <= serial_addr_counter(8);
							 
						when "00000000010010" =>	                   
							 serial_flash_in <= serial_addr_counter(7);
							 
						when "00000000010011" =>	                   
							 serial_flash_in <= serial_addr_counter(6);
							 
						when "00000000010100" =>	                   
							 serial_flash_in <= serial_addr_counter(5);
							 
						when "00000000010101" =>	                   
							 serial_flash_in <= serial_addr_counter(4);
							 
						when "00000000010110" =>	                   
							 serial_flash_in <= serial_addr_counter(3);

                  when "00000000010111" =>	                   
							 serial_flash_in <= serial_addr_counter(2);	

						when "00000000011000" =>	                   
							 serial_flash_in <= serial_addr_counter(1);	

						when "00000000011001" =>	                   
							 serial_flash_in <= serial_addr_counter(0);							 
												
						when "00000000011111" =>            -- time to send payload (switch to serial flash output)
							 serial_payload_enable <= '1';

                  when "10000000011111" =>            -- inc address counter by 1024 bytes                         
							 serial_addr_counter <= serial_addr_counter + "100000"; 					 
							 
						when others =>
							 serial_flash_in <= '0';         -- set read address lower 5 bits to zero

					end case;
				
				end if;
				
				bit_counter <= bit_counter + '1';         -- inc bit counter
				
				if(serial_addr_counter = "111111111111111") then  -- check if the whole 1MB payload is sent
                serial_payload_disable <= '1';                -- stop payload if required
					 serial_enable <= '0';
					 serial_payload_enable <= '0';
				end if;
				
			end if;
		end if;
   end process;
	
end arch_shuriken;

